@@ -13,7 +13,14 @@ use crate::common;
 use ctap_hid_fido2::public_key_credential_user_entity::PublicKeyCredentialUserEntity;
 use ctap_hid_fido2::verifier;
 
-pub fn memo(device: &FidoKeyHid, matches: &clap::ArgMatches) -> Result<()> {
+pub enum Command {
+    List,
+    Add,
+    Del(String),
+    Get(String),
+}
+
+pub fn memo(device: &FidoKeyHid, command: Command) -> Result<()> {
     if !(is_supported(device)?) {
         return Err(anyhow!(
             "This authenticator is not supported for this functions."
@@ -21,45 +28,43 @@ pub fn memo(device: &FidoKeyHid, matches: &clap::ArgMatches) -> Result<()> {
     }
 
     // Title
-    if matches.is_present("add") {
-        println!("Add a memo.");
-    } else if matches.is_present("del") {
-        println!("Delete a memo.");
-    } else if matches.is_present("list") {
-        println!("List All memos.");
-    //} else if matches.is_present("get") {
-    //    println!("Get a memo.");
-    } else {
-        println!("Get a memo.");
+    match command {
+        Command::Add => println!("Add a memo."),
+        Command::List => println!("List All memo."),
+        Command::Del(_) => println!("Delete a memo."),
+        Command::Get(_) => println!("Get a memo."),
     }
 
     let pin = common::get_pin();
     let rpid = "ctapcli";
 
     // main
-    if matches.is_present("add") {
-        let tag = common::get_input_with_message("tag:");
-        add(device, &tag, &pin, rpid)?;
-    } else if matches.is_present("del") {
-        let mut values = matches.values_of("del").unwrap();
-        let tag = values.next().unwrap();
-        del(device, tag, &pin, rpid)?;
-    } else if matches.is_present("list") {
-        list(device, &pin, rpid)?;
-    } else if matches.is_present("get") {
-        let mut values = matches.values_of("get").unwrap();
-        let tag = values.next().unwrap();
-        get(device, tag, &pin, rpid)?;
-    } else {
-        list(device, &pin, rpid)?;
-        let tag = common::get_input_with_message("tag:");
-        get(device, &tag, &pin, rpid)?;
+    match command {
+        Command::Add => {
+            let tag = common::get_input_with_message("tag:");
+            add_tag(device, &tag, &pin, rpid)?;
+        }
+        Command::List => {
+            list_tag(device, &pin, rpid)?;
+        }
+        Command::Del(tag) => {
+            del(device, &tag, &pin, rpid)?;
+        }
+        Command::Get(tag) => {
+            if tag.is_empty() {
+                list_tag(device, &pin, rpid)?;
+                let tag = common::get_input_with_message("tag:");
+                get(device, &tag, &pin, rpid)?;
+            } else {
+                get(device, &tag, &pin, rpid)?;
+            }
+        }
     }
 
     Ok(())
 }
 
-fn add(device: &FidoKeyHid, tag: &str, pin: &str, rpid: &str) -> Result<()> {
+fn add_tag(device: &FidoKeyHid, tag: &str, pin: &str, rpid: &str) -> Result<()> {
     if search_cred(device, pin, rpid, tag.as_bytes())?.is_none() {
         let memo = common::get_input_with_message("memo:");
 
@@ -90,7 +95,7 @@ fn del(device: &FidoKeyHid, tag: &str, pin: &str, rpid: &str) -> Result<()> {
     Ok(())
 }
 
-fn list(device: &FidoKeyHid, pin: &str, rpid: &str) -> Result<()> {
+fn list_tag(device: &FidoKeyHid, pin: &str, rpid: &str) -> Result<()> {
     let rps = get_rps(device, Some(pin))?;
     let mut rps = rps
         .iter()
@@ -135,7 +140,7 @@ fn get_rps(device: &FidoKeyHid, pin: Option<&str>) -> Result<Vec<Rp>> {
         Ok(rps) => Ok(rps),
         Err(e) => {
             // 0x2E CTAP2_ERR_NO_CREDENTIALS is not error
-            if e.to_string().find("0x2E").is_some() {
+            if e.to_string().contains("0x2E") {
                 Ok(vec![])
             } else {
                 Err(e)
@@ -146,7 +151,7 @@ fn get_rps(device: &FidoKeyHid, pin: Option<&str>) -> Result<Vec<Rp>> {
     //ctap_hid_fido2::credential_management_enumerate_rps(&CFG, pin)
 }
 
-fn search_cred(
+pub fn search_cred(
     device: &FidoKeyHid,
     pin: &str,
     rpid: &str,
