@@ -6,7 +6,7 @@ use ctap_hid_fido2::{
         AssertionExtension as Gext, CredentialExtension as Mext, CredentialSupportedKeyType,
         GetAssertionArgsBuilder, MakeCredentialArgsBuilder,
     },
-    get_fidokey_devices, verifier, Cfg, FidoKeyHid, FidoKeyHidFactory,
+    get_fidokey_devices, util, verifier, Cfg, FidoKeyHid, FidoKeyHidFactory,
 };
 
 fn main() -> Result<()> {
@@ -55,6 +55,10 @@ fn builder_pattern_sample(device: &FidoKeyHid, rpid: &str, pin: &str) -> Result<
 
     with_hmac(device, rpid, pin).unwrap_or_else(|err| eprintln!("Error => {}\n", err));
 
+    with_large_blob_key(device, rpid, pin).unwrap_or_else(|err| eprintln!("Error => {}\n", err));
+
+    with_min_pin_length_ex(device, rpid, pin).unwrap_or_else(|err| eprintln!("Error => {}\n", err));
+
     without_pin(device, rpid).unwrap_or_else(|err| eprintln!("Error => {}\n", err));
 
     Ok(())
@@ -236,6 +240,19 @@ fn with_hmac(device: &FidoKeyHid, rpid: &str, pin: &str) -> Result<()> {
 
     let attestation = device.make_credential_with_args(&make_credential_args)?;
     println!("-- Register Success");
+    let find = attestation.extensions.iter().find(|it| {
+        if let Mext::HmacSecret(_) = it {
+            true
+        } else {
+            false
+        }
+    });
+    if let Some(Mext::HmacSecret(is_hmac_secret)) = find {
+        println!("--- HMAC Secret = {:?}", is_hmac_secret.unwrap());
+    } else {
+        println!("--- HMAC Secret Not Found");
+    }
+
     debug!("Attestation");
     debug!("{}", attestation);
 
@@ -259,6 +276,21 @@ fn with_hmac(device: &FidoKeyHid, rpid: &str, pin: &str) -> Result<()> {
 
     let assertions = device.get_assertion_with_args(&get_assertion_args)?;
     println!("-- Authenticate Success");
+    let find = assertions[0].extensions.iter().find(|it| {
+        if let Gext::HmacSecret(_) = it {
+            true
+        } else {
+            false
+        }
+    });
+    if let Some(Gext::HmacSecret(hmac_secret)) = find {
+        println!(
+            "--- HMAC Secret = {}",
+            util::to_hex_str(&hmac_secret.unwrap())
+        )
+    } else {
+        println!("--- HMAC Secret Not Found");
+    }
     debug!("Assertion");
     debug!("{}", assertions[0]);
 
@@ -332,6 +364,128 @@ fn without_pin(device: &FidoKeyHid, rpid: &str) -> Result<()> {
     Ok(())
 }
 
+fn with_large_blob_key(device: &FidoKeyHid, rpid: &str, pin: &str) -> Result<()> {
+    println!("----- with large_blob_key -----");
+
+    println!("- Register");
+    let challenge = verifier::create_challenge();
+    let ext = Mext::LargeBlobKey((Some(true), None));
+
+    let make_credential_args = MakeCredentialArgsBuilder::new(&rpid, &challenge)
+        .pin(pin)
+        .extensions(&vec![ext])
+        .build();
+
+    let attestation = device.make_credential_with_args(&make_credential_args)?;
+    println!("-- Register Success");
+    let find = attestation.extensions.iter().find(|it| {
+        if let Mext::LargeBlobKey((_, _)) = it {
+            true
+        } else {
+            false
+        }
+    });
+    if let Some(Mext::LargeBlobKey((_, large_blob_key))) = find {
+        println!(
+            "--- Large Blob Key = {}",
+            util::to_hex_str(&large_blob_key.clone().unwrap())
+        );
+    } else {
+        println!("--- Large Blob Key Not Found");
+    }
+    debug!("Attestation");
+    debug!("{}", attestation);
+
+    println!("-- Verify Attestation");
+    let verify_result = verifier::verify_attestation(rpid, &challenge, &attestation);
+    if verify_result.is_success {
+        println!("-- Verify Attestation Success");
+    } else {
+        println!("-- ! Verify Attestation Failed");
+    }
+
+    println!("- Authenticate");
+    let challenge = verifier::create_challenge();
+    let ext = Gext::LargeBlobKey((Some(true), None));
+
+    let get_assertion_args = GetAssertionArgsBuilder::new(rpid, &challenge)
+        .pin(pin)
+        .credential_id(&verify_result.credential_id)
+        .extensions(&vec![ext])
+        .build();
+
+    let assertions = device.get_assertion_with_args(&get_assertion_args)?;
+    println!("-- Authenticate Success");
+    let find = assertions[0].extensions.iter().find(|it| {
+        if let Gext::LargeBlobKey((_, _)) = it {
+            true
+        } else {
+            false
+        }
+    });
+    if let Some(Gext::LargeBlobKey((_, large_blob_key))) = find {
+        println!(
+            "--- Large Blob Key = {}",
+            util::to_hex_str(&large_blob_key.clone().unwrap())
+        );
+    } else {
+        println!("--- Large Blob Key Not Found");
+    }
+    debug!("Assertion");
+    debug!("{}", assertions[0]);
+
+    println!("-- Verify Assertion");
+    let is_success = verifier::verify_assertion(
+        rpid,
+        &verify_result.credential_publickey_der,
+        &challenge,
+        &assertions[0],
+    );
+    if is_success {
+        println!("-- Verify Assertion Success");
+    } else {
+        println!("-- ! Verify Assertion Failed");
+    }
+
+    println!();
+    Ok(())
+}
+
+fn with_min_pin_length_ex(device: &FidoKeyHid, rpid: &str, pin: &str) -> Result<()> {
+    println!("----- with Min Pin Length Extension -----");
+    println!("       - Get Current Min Pin Length");
+    println!("       - Need Set Config Min Pin Length RPIDs [{}]", rpid);
+
+    println!("- Register");
+    let challenge = verifier::create_challenge();
+    let ext = Mext::MinPinLength((Some(true), None));
+
+    let make_credential_args = MakeCredentialArgsBuilder::new(&rpid, &challenge)
+        .pin(pin)
+        .extensions(&vec![ext])
+        .build();
+
+    let attestation = device.make_credential_with_args(&make_credential_args)?;
+    println!("-- Register Success");
+    let find = attestation.extensions.iter().find(|it| {
+        if let Mext::MinPinLength((_, _)) = it {
+            true
+        } else {
+            false
+        }
+    });
+    if let Some(Mext::MinPinLength((_, min_pin_length))) = find {
+        println!("--- Min Pin Length = {:?}", min_pin_length);
+    } else {
+        println!("--- Min Pin Length Not Found");
+    }
+    debug!("Attestation");
+    debug!("{}", attestation);
+
+    println!();
+    Ok(())
+}
+
 //
 // Legacy Pattern Sample
 //
