@@ -2,11 +2,9 @@
 // cargo test -- --test-threads=1
 //
 
-use ctap_hid_fido2::str_buf::StrBuf;
-use ctap_hid_fido2::util;
 use ctap_hid_fido2::*;
-use ring::digest;
-use std::convert::TryFrom;
+use fidokey::get_info::{InfoOption, InfoParam};
+use fidokey::MakeCredentialArgsBuilder;
 
 #[test]
 fn test_get_hid_devices() {
@@ -16,19 +14,22 @@ fn test_get_hid_devices() {
 
 #[test]
 fn test_wink() {
-    wink(&Cfg::init()).unwrap();
+    let device = FidoKeyHidFactory::create(&Cfg::init()).unwrap();
+    device.wink().unwrap();
     assert!(true);
 }
 
 #[test]
 fn test_get_info() {
-    get_info(&Cfg::init()).unwrap();
+    let device = FidoKeyHidFactory::create(&Cfg::init()).unwrap();
+    device.get_info().unwrap();
     assert!(true);
 }
 
 #[test]
 fn test_get_info_u2f() {
-    match ctap_hid_fido2::enable_info_param(&Cfg::init(), &InfoParam::VersionsU2Fv2) {
+    let device = FidoKeyHidFactory::create(&Cfg::init()).unwrap();
+    match device.enable_info_param(&InfoParam::VersionsU2Fv2) {
         Ok(result) => {
             if !result {
                 // Skip
@@ -38,13 +39,14 @@ fn test_get_info_u2f() {
         Err(_) => assert!(false),
     };
 
-    get_info_u2f(&Cfg::init()).unwrap();
+    device.get_info_u2f().unwrap();
     assert!(true);
 }
 
 #[test]
 fn test_client_pin_get_retries() {
-    let retry = get_pin_retries(&Cfg::init());
+    let device = FidoKeyHidFactory::create(&Cfg::init()).unwrap();
+    let retry = device.get_pin_retries();
     println!("- retries = {:?}", retry);
     assert!(true);
 }
@@ -56,27 +58,53 @@ fn test_make_credential_with_pin_non_rk() {
     let challenge = b"this is challenge".to_vec();
     let pin = "1234";
 
-    let att = make_credential(&Cfg::init(), rpid, &challenge, Some(pin)).unwrap();
+    let device = FidoKeyHidFactory::create(&Cfg::init()).unwrap();
+    let att = device.make_credential(rpid, &challenge, Some(pin)).unwrap();
     println!("Attestation");
     println!("{}", att);
 
-    let ass = get_assertion(
-        &Cfg::init(),
-        rpid,
-        &challenge,
-        &att.credential_descriptor.id,
-        Some(pin),
-    )
-    .unwrap();
+    let ass = device
+        .get_assertion(rpid, &challenge, &[att.credential_descriptor.id], Some(pin))
+        .unwrap();
     println!("Assertion");
     println!("{}", ass);
 
     assert!(true);
 }
 
+#[test]
+fn test_make_credential_with_pin_non_rk_exclude_authenticator() {
+    // parameter
+    let rpid = "test.com";
+    let challenge = b"this is challenge".to_vec();
+    let pin = "1234";
+
+    let device = FidoKeyHidFactory::create(&Cfg::init()).unwrap();
+
+    let make_credential_args = MakeCredentialArgsBuilder::new(&rpid, &challenge)
+        .pin(pin)
+        .build();
+
+    let att = device
+        .make_credential_with_args(&make_credential_args)
+        .unwrap();
+
+    let verify_result = verifier::verify_attestation(rpid, &challenge, &att);
+    assert!(verify_result.is_success);
+
+    let make_credential_args = MakeCredentialArgsBuilder::new(&rpid, &challenge)
+        .pin(pin)
+        .exclude_authenticator(&verify_result.credential_id)
+        .build();
+
+    let result = device.make_credential_with_args(&make_credential_args);
+    assert!(result.is_err());
+}
+
 #[test]
 fn test_credential_management_get_creds_metadata() {
-    match ctap_hid_fido2::enable_info_param(&Cfg::init(), &InfoParam::VersionsFido21Pre) {
+    let device = FidoKeyHidFactory::create(&Cfg::init()).unwrap();
+    match device.enable_info_param(&InfoParam::VersionsFido21Pre) {
         Ok(result) => {
             if !result {
                 // Skip
@@ -87,7 +115,7 @@ fn test_credential_management_get_creds_metadata() {
     };
 
     let pin = "1234";
-    match ctap_hid_fido2::credential_management_get_creds_metadata(&Cfg::init(), Some(pin)) {
+    match device.credential_management_get_creds_metadata(Some(pin)) {
         Ok(_) => assert!(true),
         Err(_) => assert!(false),
     };
@@ -95,7 +123,8 @@ fn test_credential_management_get_creds_metadata() {
 
 #[test]
 fn test_credential_management_enumerate_rps() {
-    match ctap_hid_fido2::enable_info_param(&Cfg::init(), &InfoParam::VersionsFido21Pre) {
+    let device = FidoKeyHidFactory::create(&Cfg::init()).unwrap();
+    match device.enable_info_param(&InfoParam::VersionsFido21Pre) {
         Ok(result) => {
             if !result {
                 // Skip
@@ -106,7 +135,7 @@ fn test_credential_management_enumerate_rps() {
     };
 
     let pin = "1234";
-    match ctap_hid_fido2::credential_management_enumerate_rps(&Cfg::init(), Some(pin)) {
+    match device.credential_management_enumerate_rps(Some(pin)) {
         Ok(_) => assert!(true),
         Err(_) => assert!(false),
     };
@@ -115,8 +144,10 @@ fn test_credential_management_enumerate_rps() {
 #[test]
 fn test_bio_enrollment_get_fingerprint_sensor_info() {
     let mut skip = true;
-    match ctap_hid_fido2::enable_info_option(&Cfg::init(), &InfoOption::UserVerificationMgmtPreview)
-    {
+
+    let device = FidoKeyHidFactory::create(&Cfg::init()).unwrap();
+
+    match device.enable_info_option(&InfoOption::UserVerificationMgmtPreview) {
         Ok(result) => {
             //println!("result = {:?}", result);
             if let Some(v) = result {
@@ -134,7 +165,7 @@ fn test_bio_enrollment_get_fingerprint_sensor_info() {
         return;
     };
 
-    match ctap_hid_fido2::bio_enrollment_get_fingerprint_sensor_info(&Cfg::init()) {
+    match device.bio_enrollment_get_fingerprint_sensor_info() {
         Ok(_) => assert!(true),
         Err(_) => assert!(false),
     };
@@ -143,8 +174,10 @@ fn test_bio_enrollment_get_fingerprint_sensor_info() {
 #[test]
 fn test_bio_enrollment_enumerate_enrollments() {
     let mut skip = true;
-    match ctap_hid_fido2::enable_info_option(&Cfg::init(), &InfoOption::UserVerificationMgmtPreview)
-    {
+
+    let device = FidoKeyHidFactory::create(&Cfg::init()).unwrap();
+
+    match device.enable_info_option(&InfoOption::UserVerificationMgmtPreview) {
         Ok(result) => {
             if let Some(v) = result {
                 if v {
@@ -160,42 +193,8 @@ fn test_bio_enrollment_enumerate_enrollments() {
     };
 
     let pin = "1234";
-    match ctap_hid_fido2::bio_enrollment_enumerate_enrollments(&Cfg::init(), pin) {
+    match device.bio_enrollment_enumerate_enrollments(pin) {
         Ok(_) => assert!(true),
         Err(_) => assert!(false),
     };
 }
-
-#[test]
-fn test_enc_hmac_sha_256() {
-    let key_str = "this is key.";
-    let hasher = digest::digest(&digest::SHA256, &key_str.as_bytes());
-    let key = <[u8; 32]>::try_from(hasher.as_ref()).unwrap();
-
-    let message = "this is message.";
-    let sig = enc_hmac_sha_256::authenticate(&key, message.as_bytes());
-    print!("{}", StrBuf::bufh("- hmac signature", &sig));
-    assert_eq!(
-        sig,
-        util::to_str_hex("BF3D3FCFC4462CDCBEBBBC8AF82EA38B7B5ED4259B2061322C57B5CA696D6080")
-    );
-}
-
-#[test]
-fn test_enc_aes256_cbc() {
-    let key_str = "this is key.";
-    let hasher = digest::digest(&digest::SHA256, &key_str.as_bytes());
-    let key = <[u8; 32]>::try_from(hasher.as_ref()).unwrap();
-
-    let message = "this is message.";
-    let enc_data = enc_aes256_cbc::encrypt_message_str(&key, message);
-    print!("{}", StrBuf::bufh("- enc_data", &enc_data));
-    assert_eq!(
-        enc_data,
-        util::to_str_hex("37455A8392187439EFAA249617AAB5C2")
-    );
-
-    let dec_data = enc_aes256_cbc::decrypt_message_str(&key, &enc_data);
-    print!("- dec_data = {}", dec_data);
-    assert_eq!(dec_data, message);
-}
